/********************************************************************* 
-- S+ sparse GE solver (version 1.1) --

Copyright (C) 1999 by University of California at Santa Barbara. 
Copyright (C) 2005 by University of Rochester. 
All rights reserved.

S+ team contact: Kai Shen <kshen@cs.rochester.edu>, 
                 Tao Yang <tyang@cs.ucsb.edu>  

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose, without fee, and without written agreement
is hereby granted, provided that the above copyright notice and the
following paragraph appear in all copies of this software.

This software is provided ``as is'', without any warranty, including 
any implied warranty for merchantability or fitness for a particular
purpose. Under no circumstances shall the University of California at 
Santa Barbara, the University of Rochester, or their agents be liable 
for any use of, misuse of, or inability to use this software, including 
incidental and consequential damages.
************************************************************************/

/*
 * Splus.h - include file for using S+ library
 *
 */


#ifndef __SPLUS_H_
#define __SPLUS_H_

#ifdef SINGLE_PRECISION
typedef float ELEMTYPE;
#else
typedef double ELEMTYPE;
#endif

/* struture of input sparse matrix */
typedef struct {
  int order;		/* order of the sparse matrix */
  int numNZ;		/* number of nonzeros in the sparse matrix */
  ELEMTYPE *coe;	/* the nonzero coefficients in a column-major order */
  int *rowIndex;	/* the row indices of the nonzeros in the same order as coe */
  int *colStart;	/* starting index of each column in array coe,
			   the last element is numNZ which serves as a guard */
} SP_SparseMatrix;

/* structure of dense matrix */
typedef struct {
  int rowNum;		/* row number of the dense matrix */
  int colNum;		/* column number of the dense matrix */
  ELEMTYPE *element;	/* element array */
} SP_DenseMatrix;


/* generate a input matrix from a ij-value data file */
SP_SparseMatrix *SP_ReadMatrixFromIJ(const char*);

/* generate a input matrix from a Harwell Boeing RUA-type data file */
SP_SparseMatrix* SP_ReadMatrixFromHB(const char*);

/* ordering routine */
void SP_Ordering(int, SP_SparseMatrix*, int**, int**);

/* Linear system solver */
void SP_Solve(SP_SparseMatrix*, SP_DenseMatrix*, int*, int*, int, int);

/* detroy and free the data struture for a sparse matrix */
void SP_DestroySparseMatrix(SP_SparseMatrix*);

/* set user defined error handler */
/* not supported in this version because of updates in SGI compiler */
/* SP_ERRHANDLER_T SP_SetErrorHandler(SP_ERRHANDLER_T f); */

/* set block size limit, returns 1 if successful, 0 otherwise */
int SP_SetParam_BlkSize(int);

/* get block size limit */
int SP_GetParam_BlkSize();

/* set relax parameter for amalgamation, returns 1 if successful, 0 otherwise */
int SP_SetParam_Relax(double);

/* get relax parameter for amalgamation */
double SP_GetParam_Relax();

/* set the parameter for threshold pivoting, returns 1 if successful, 0 otherwise */
int SP_SetParam_ThrholdPivot(double);

/* get the parameter for threshold pivoting */
double SP_GetParam_ThrholdPivot();

/* column-by-column pivoting, 
   large diagonal batching pivoting, 
   and speculative batch pivoting */
enum {SP_PIVOTING_COLBYCOL=0, SP_PIVOTING_LD, SP_PIVOTING_SBP, SP_PIVOTING_COUNT};

/* set the parameter for batch pivoting, returns 1 if successful, 0 otherwise */
int SP_SetParam_BatchPivot(int);

/* get the parameter for batch pivoting */
int SP_GetParam_BatchPivot();

#endif
